import argparse
import logging
import os
import re
import sys
from collections.abc import Iterable
from functools import wraps
from pathlib import Path
from subprocess import run

# import ipdb
from IPython.core import ultratb
from IPython.terminal.debugger import TerminalPdb
from pychromecast import discovery
from rich import inspect, print
from rich.logging import RichHandler

# sys.breakpointhook = ipdb.set_trace


def parse_args(default_chromecast="Xylo and Orchestra"):
    parser = argparse.ArgumentParser()
    parser.add_argument("db")

    # TODO: maybe try https://dba.stackexchange.com/questions/43415/algorithm-for-finding-the-longest-prefix
    parser.add_argument("-O", "--play-in-order", action="count", default=0)
    parser.add_argument("-S", "--skip")
    parser.add_argument("-u", "--sort")
    parser.add_argument("-w", "--where")
    parser.add_argument("--only-video", action="store_true")

    parser.add_argument("-d", "--duration", type=int)
    parser.add_argument("-dM", "--max-duration", type=int)
    parser.add_argument("-dm", "--min-duration", type=int)

    parser.add_argument("-s", "--search")
    parser.add_argument("-E", "--exclude")

    parser.add_argument("-cast-to", "--chromecast-device", default=default_chromecast)
    parser.add_argument("-cast", "--chromecast", action="store_true")
    parser.add_argument("-wl", "--with-local", action="store_true")

    parser.add_argument("-f", "--prefix", default="", help="sshfs root prefix")

    parser.add_argument("-z", "--size", type=int)
    parser.add_argument("--max-size", type=int)
    parser.add_argument("--min-size", type=int)

    parser.add_argument("-p", "--print", action="store_true")
    parser.add_argument("-pq", "--printquery", action="store_true")
    parser.add_argument("-L", "--limit", type=int)
    parser.add_argument("-a", "--aggregate", action="store_true")
    parser.add_argument("-name", "--filename", action="store_true")

    parser.add_argument("-t", "--time-limit", type=int)
    parser.add_argument("-vlc", "--vlc", action="store_true")
    parser.add_argument("--force-transcode", action="store_true")

    parser.add_argument("-k", "--keep", action="store_true")
    parser.add_argument("-mv", "--move")
    parser.add_argument("--delete", action="store_true")

    parser.add_argument("-v", "--verbose", action="count", default=0)
    args = parser.parse_args()

    if args.limit is None:
        args.limit = 1
        if args.print:
            args.limit = 100
        if args.aggregate:
            args.limit = 9999999999999

    YEAR_MONTH = lambda var: f"cast(strftime('%Y%m',datetime({var} / 1000000000, 'unixepoch')) as int)"
    if args.sort:
        args.sort = args.sort.replace("time_created", YEAR_MONTH("time_created"))
        args.sort = args.sort.replace("time_modified", YEAR_MONTH("time_modified"))
        args.sort = args.sort.replace("random", "random()")
        args.sort = args.sort.replace("priority", "round(duration / size,7)")
        args.sort = args.sort.replace("sub", "has_sub")

    if args.where:
        args.where = args.where.replace("sub", "has_sub")

    return args


def remove_media(con, filename):
    con.execute("delete from media where filename = ?", (str(filename),))
    con.commit()


def stop():
    exit(255)  # use nonzero code to stop shell repeat


def get_video_files(path, audio=False):
    FFMPEG_DEMUXERS = "str|aa|aac|aax|ac3|acm|adf|adp|dtk|ads|ss2|adx|aea|afc|aix|al|ape|apl|mac|aptx|aptxhd|aqt|ast|obu|avi|avr|avs|avs2|avs3|bfstm|bcstm|binka|bit|bmv|brstm|cdg|cdxl|xl|c2|302|daud|str|adp|dav|dss|dts|dtshd|dv|dif|cdata|eac3|paf|fap|flm|flac|flv|fsb|fwse|g722|722|tco|rco|g723_1|g729|genh|gsm|h261|h26l|h264|264|avc|hca|hevc|h265|265|idf|ifv|cgi|ipu|sf|ircam|ivr|kux|669|abc|amf|ams|dbm|dmf|dsm|far|it|mdl|med|mid|mod|mt2|mtm|okt|psm|ptm|s3m|stm|ult|umx|xm|itgz|itr|itz|mdgz|mdr|mdz|s3gz|s3r|s3z|xmgz|xmr|xmz|669|amf|ams|dbm|digi|dmf|dsm|dtm|far|gdm|ice|imf|it|j2b|m15|mdl|med|mmcmp|mms|mo3|mod|mptm|mt2|mtm|nst|okt|plm|ppm|psm|pt36|ptm|s3m|sfx|sfx2|st26|stk|stm|stp|ult|umx|wow|xm|xpk|flv|dat|lvf|m4v|mkv|mk3d|mka|mks|webm|mca|mcc|mjpg|mjpeg|mpo|j2k|mlp|mods|moflex|mov|mp4|m4a|3gp|3g2|mj2|psp|m4b|ism|ismv|isma|f4v|mp2|m2a|mpa|mpc|mjpg|mpl2|msf|mtaf|ul|musx|mvi|mxg|v|nist|sph|nsp|nut|obu|ogg|oma|omg|aa3|pjs|pvf|yuv|cif|qcif|rgb|rt|rsd|rsd|rso|sw|sb|sami|sbc|msbc|sbg|scc|sdr2|sds|sdx|ser|sga|shn|vb|son|imx|sln|mjpg|stl|sup|svag|svs|tak|thd|tta|ans|art|asc|diz|ice|vt|ty|ty+|uw|ub|v210|yuv10|vag|vc1|rcv|viv|vpk|vqf|vql|vqe|wsd|xmv|xvag|yop|y4m"
    if audio:
        FFMPEG_DEMUXERS += "|opus|oga|mp3"

    video_files = []

    for f in Path(path).resolve().rglob("*"):
        if f.is_file() and (f.suffix.lower()[1:] in FFMPEG_DEMUXERS.split("|")):
            video_files.append(str(f))

    return video_files


def chunks(lst, n):
    for i in range(0, len(lst), n):
        yield lst[i : i + n]


def run_once(f):
    @wraps(f)
    def wrapper(*args, **kwargs):
        if not f.has_run:
            result = f(*args, **kwargs)
            f.has_run = True
            return result

    f.has_run = False
    return wrapper


@run_once
def argparse_log():
    parser = argparse.ArgumentParser(add_help=False)
    parser.add_argument("-v", "--verbose", action="count", default=0)
    args, _unknown = parser.parse_known_args()
    # print(args)

    try:
        if args.verbose > 0 and os.getpgrp() == os.tcgetpgrp(sys.stdout.fileno()):
            sys.excepthook = ultratb.FormattedTB(
                mode="Context",
                color_scheme="Neutral",
                call_pdb=1,
                debugger_cls=TerminalPdb,
            )
        else:
            pass
    except:
        pass

    log_levels = [logging.WARNING, logging.INFO, logging.DEBUG]
    logging.root.handlers = []  # clear any existing handlers
    logging.basicConfig(
        level=log_levels[min(len(log_levels) - 1, args.verbose)],
        format="%(message)s",
        datefmt="[%X]",
        handlers=[RichHandler()],
    )
    return logging.getLogger()


log = argparse_log()


def cmd(command, strict=True, cwd=None, quiet=False):
    EXP_FILTER = re.compile(
        "|".join(
            [
                r".*Stream #0:0.*Audio: opus, 48000 Hz, .*, fltp",
                r".*Metadata:",
            ]
        ),
        re.IGNORECASE,
    )

    def filter_output(string):
        filtered_strings = []
        for s in string.strip().splitlines():
            if not EXP_FILTER.match(s):
                filtered_strings.append(s)

        return "\n".join(list(filter(None, filtered_strings)))

    def print_std(r_std):
        s = filter_output(r_std)
        if not quiet and len(s) > 0:
            print(s)
        return s

    r = run(command, capture_output=True, text=True, shell=True, cwd=cwd, preexec_fn=os.setpgrp)
    # TODO Windows support: creationflags=subprocess.DETACHED_PROCESS | subprocess.CREATE_NEW_PROCESS_GROUP
    log.debug(r.args)
    r.stdout = print_std(r.stdout)
    r.stderr = print_std(r.stderr)
    if r.returncode != 0:
        log.info(f"ERROR {r.returncode}")
        if strict:
            raise Exception(f"[{command}] exited {r.returncode}")

    return r


def conditional_filter(args):
    B_TO_MB = 1024 * 1024
    size_mb = 0
    if args.size:
        size_mb = args.size * B_TO_MB

    SEC_TO_M = 60
    duration_m = 0
    if args.duration:
        duration_m = args.duration * SEC_TO_M

    cf = f"""duration IS NOT NULL and size IS NOT NULL
    {f'and duration >= {args.min_duration * SEC_TO_M}' if args.min_duration else ''}
    {f'and {args.max_duration * SEC_TO_M} >= duration' if args.max_duration else ''}
    {f'and {duration_m + (duration_m /10)} >= duration and duration >= {duration_m - (duration_m /10)}' if args.duration else ''}
    {f'and size >= {args.min_size * B_TO_MB}' if args.min_size else ''}
    {f'and {args.max_size * B_TO_MB} >= size' if args.max_size else ''}
    {f'and {size_mb + (size_mb /10)} >= size and size >= {size_mb - (size_mb /10)}' if args.size else ''}
    """

    if args.where:
        cf += " and " + args.where

    return " ".join(cf.splitlines())


def compile_query(query, *args):
    if len(args) == 1 and (not args[0]):
        number_of_arguments = 0

    number_of_question_marks = query.count("?")
    number_of_arguments = len(args)
    if number_of_arguments != number_of_question_marks:
        return f"Number of bindings mismatched. The query uses {number_of_question_marks}, but {number_of_arguments} binded parameters."

    for a in args:
        query = query.replace("?", "'" + str(a) + "'", 1)

    return query


def print_query(bindings, query):
    print(re.sub(r"\n\s+", r"\n", compile_query(query, *bindings)))


def single_column_tolist(array_of_half_tuplets, column_name=1):
    return list(
        map(
            lambda x: x[column_name],
            array_of_half_tuplets,
        )
    )


def get_ordinal_media(con, args, filename: Path, sql_filter):
    similar_videos = []
    testname = str(filename)

    total_media = con.execute("select count(*) val from media").fetchone()[0]
    while len(similar_videos) < 2:
        remove_groups = re.split(r"([\W]+|\s+|Ep\d+|x\d+|\.\d+)", testname)
        log.debug(remove_groups)
        remove_chars = ""
        remove_chars_i = 1
        while len(remove_chars) < 1:
            remove_chars += remove_groups[-remove_chars_i]
            remove_chars_i += 1

        newtestname = testname[: -len(remove_chars)]
        log.debug(f"Matches for '{newtestname}':")

        if testname in ["" or newtestname]:
            return filename

        testname = newtestname
        query = f"""SELECT filename FROM media
            WHERE filename like ?
                and {'1=1' if (args.play_in_order > 2) else sql_filter}
            ORDER BY filename
            LIMIT 1000
            """
        bindings = ("%" + testname + "%",)
        if args.printquery:
            print_query(bindings, query)
            stop()

        similar_videos = single_column_tolist(con.execute(query, bindings).fetchall(), "filename")  # type: ignore
        log.debug(similar_videos)

        if len(similar_videos) > 999 or len(similar_videos) == total_media:
            return filename

        commonprefix = os.path.commonprefix(similar_videos)
        log.debug(commonprefix)
        if len(Path(commonprefix).name) < 3:
            log.debug("Using commonprefix")
            return filename

    return similar_videos[0]


def get_ip_of_chromecast(device_name):
    cast_infos, browser = discovery.discover_listed_chromecasts(friendly_names=[device_name])
    browser.stop_discovery()
    if len(cast_infos) == 0:
        print("Target chromecast device not found")
        exit(53)

    return cast_infos[0].host


def flatten(xs):
    for x in xs:
        if isinstance(x, Iterable) and not isinstance(x, (str, bytes)):
            yield from flatten(x)
        elif isinstance(x, bytes):
            yield x.decode("utf-8")
        else:
            yield x


def remove_None(kwargs):
    return {k: v for k, v in kwargs.items() if v is not None}


_RE_COMBINE_WHITESPACE = re.compile(r"\s+")


def conform(list_):
    if not isinstance(list_, list):
        list_ = [list_]
    list_ = flatten(list_)
    list_ = list(filter(None, list_))
    return list_


def combine(*list_):
    list_ = conform(list_)
    if len(list_) == 0:
        return None

    no_comma = sum([s.split(",") for s in list_], [])
    no_semicol = sum([s.split(";") for s in no_comma], [])
    no_double_space = [_RE_COMBINE_WHITESPACE.sub(" ", s).strip() for s in no_semicol]
    no_unknown = [x for x in no_double_space if x.lower() not in ["unknown", "none", "und", ""]]

    no_duplicates = list(set(no_unknown))
    return ";".join(no_duplicates)


def safe_unpack(*list_, idx=0):
    list_ = conform(list_)
    if len(list_) == 0:
        return None

    try:
        return list_[idx]
    except IndexError:
        return None



import subprocess
from pathlib import Path
from shlex import quote
from time import sleep

from rich import inspect

from .db import sqlite_con
from .utils import cmd, conditional_filter, get_ordinal_media, log, parse_args, remove_media, stop


def play_mpv(args, audio_path: Path):
    mpv_options = (
        "--input-ipc-server=/tmp/mpv_socket --no-video --replaygain=track --volume=100 --keep-open=no --term-osd-bar"
    )
    quoted_next_audio = quote(str(audio_path))

    try:
        print(cmd(f"ffprobe -hide_banner -loglevel info {quoted_next_audio}", quiet=True).stderr)
    except:
        print(quoted_next_audio)

    if args.chromecast:
        Path("/tmp/mpcatt_playing").write_text(quoted_next_audio)

        if not args.with_local:
            cmd("touch /tmp/sub.srt")
            cmd(f"catt -d '{args.chromecast_device}' cast -s /tmp/sub.srt {quoted_next_audio}")
        else:
            cast_process = subprocess.Popen(["catt", "-d", args.chromecast_device, "cast", audio_path])
            sleep(1.174)  # imperfect lazy sync; I use keyboard shortcuts to send `set speed` commands to mpv for resync
            # kde-inhibit --power
            cmd(f"mpv {mpv_options} -- {quoted_next_audio}")
            cast_process.communicate()  # wait for chromecast to stop (so that I can tell any chromecast to pause)
            sleep(3.0)  # give chromecast some time to breathe
    else:
        cmd(f"mpv {mpv_options} -- {quoted_next_audio}", quiet=True)


def listen(args):
    con = sqlite_con(args.db)

    bindings = {}
    if args.search:
        bindings["search"] = "%" + args.search.replace(" ", "%").replace("%%", " ") + "%"
    if args.exclude:
        bindings["exclude"] = "%" + args.exclude.replace(" ", "%").replace("%%", " ") + "%"

    sql_filter = conditional_filter(args)

    search_string = """and (
        filename like :search
        OR mood like :search
        OR genre like :search
        OR year like :search
        OR bpm like :search
        OR key like :search
        OR time like :search
        OR decade like :search
        OR categories like :search
        OR city like :search
        OR country like :search
        OR description like :search
        OR album like :search
        OR title like :search
        OR artist like :search
    )"""

    exclude_string = """and (
        filename not like :exclude
        OR mood not like :exclude
        OR genre not like :exclude
        OR year not like :exclude
        OR bpm not like :exclude
        OR key not like :exclude
        OR time not like :exclude
        OR decade not like :exclude
        OR categories not like :exclude
        OR city not like :exclude
        OR country not like :exclude
        OR description not like :exclude
        OR album not like :exclude
        OR title not like :exclude
        OR artist not like :exclude
    )"""

    query = f"""
    SELECT filename, duration / size AS seconds_per_byte, size
    FROM media
    WHERE 1=1
    {search_string if args.search else ''}
    {exclude_string if args.exclude else ''}
    {"" if args.search else 'and listen_count = 0'}
    and {sql_filter}
    ORDER BY
        listen_count asc nulls first,
        {args.sort + ',' if args.sort else ''}
        {'filename,' if args.search and (args.play_in_order > 0) else ''}
        seconds_per_byte ASC
    limit 1 OFFSET {args.skip if args.skip else 0}
    """

    if args.printquery:
        print(query)
        stop()

    next_audio = dict(con.execute(query, bindings).fetchone())
    next_audio = Path(next_audio["filename"])

    # limit to audiobook since normal music does not get deleted so only the first track would ever be played
    if "audiobook" in str(next_audio):
        next_audio = Path(get_ordinal_media(con, args, next_audio, sql_filter))

    if args.print:
        print(next_audio)
        stop()

    if not next_audio.exists():
        print("Removing orphaned metadata", next_audio)
        remove_media(con, next_audio)
    else:
        quoted_next_audio = quote(str(next_audio))

        if args.move:
            keep_path = str(Path(args.move))
            cmd(f"mv {quoted_next_audio} {quote(keep_path)}")
        else:
            play_mpv(args, next_audio)
            if args.delete or "audiobook" in quoted_next_audio.lower():
                cmd(f"trash-put {quoted_next_audio}", strict=False)
                remove_media(con, next_audio)

    con.execute("update media set listen_count = listen_count +1 where filename = ?", (str(next_audio),))
    con.commit()


def main():
    args = parse_args()

    try:
        listen(args)
    finally:
        if args.chromecast:
            cmd("rm /tmp/mpcatt_playing", strict=False)


if __name__ == "__main__":
    main()



import json
import math
import os
import re
import textwrap
from datetime import timedelta
from pathlib import Path
from shlex import quote

import humanize
import pandas as pd
import sqlite_utils
from rich import inspect
from rich.prompt import Confirm
from tabulate import tabulate

from .db import sqlite_con
from .utils import (
    cmd,
    conditional_filter,
    get_ip_of_chromecast,
    get_ordinal_media,
    log,
    parse_args,
    print_query,
    remove_media,
    stop,
)


def play_mpv(args, video_path: Path):
    mpv = "mpv"
    mpv_options = "--fs --force-window=yes --terminal=no"
    vlc = "vlc"
    quoted_video_path = quote(str(video_path))
    is_WSL = cmd('grep -qEi "(Microsoft|WSL)" /proc/version', strict=False).returncode == 0
    if is_WSL:
        mpv = "PULSE_SERVER=tcp:localhost mpv"
        vlc = "PULSE_SERVER=tcp:localhost cvlc"

    if args.chromecast:
        subs = json.loads(
            cmd(f"ffprobe -loglevel error -select_streams s -show_entries stream -of json {quoted_video_path}").stdout
        )["streams"]

        subtitles_file = None
        if len(subs) > 0:
            db = sqlite_utils.Database(memory=True)
            db["subs"].insert_all(subs, pk="index")  # type: ignore
            subtitle_index = db.execute_returning_dicts(
                """select "index" from subs
                order by
                    lower(tags) like "%eng%" desc
                    , lower(tags) like "%dialog%" desc
                limit 1"""
            )[0]["index"]
            log.debug(f"Using subtitle {subtitle_index}")

            subtitles_file = cmd("mktemp --suffix=.vtt --dry-run").stdout.strip()
            cmd(
                f'ffmpeg -loglevel warning -txt_format text -i {quoted_video_path} -map "0:{subtitle_index}" "{subtitles_file}"'
            )

        if args.vlc:
            watched = cmd(
                f"{vlc} --sout '#chromecast' --sout-chromecast-ip={args.cc_ip} --demux-filter=demux_chromecast {'--sub-file='+subtitles_file if subtitles_file else ''} {quoted_video_path}"
            )
        else:
            watched = cmd(
                f"catt -d '{args.chromecast_device}' cast {quoted_video_path} {'--subtitles '+subtitles_file if subtitles_file else ''}"
            )

        if subtitles_file:
            Path(subtitles_file).unlink(missing_ok=True)

        if "Heartbeat timeout, resetting connection" in watched.stderr:
            raise Exception("Media is possibly partially unwatched")

        if watched.stderr == "":
            raise Exception("catt does not exit nonzero? but something might have gone wrong")

        return  # end of chromecast

    has_sub = (
        cmd(
            f"</dev/null ffmpeg -c copy -map 0:s:0 -frames:s 1 -f null - -v 0 -i {quoted_video_path}",
            strict=False,
            quiet=True,
        ).returncode
        == 0
    )
    if not has_sub:
        mpv_options += " --speed=1.7"
    else:
        mpv_options += " --speed=1"

    cmd(f"{mpv} {mpv_options} {quoted_video_path}")


def keep_video(video: Path):
    kp = re.match(".*?/mnt/d/(.*?)/", str(video))
    if kp:
        keep_path = str(Path(kp[0], "keep/"))
    else:
        keep_path = str(video.parent / "keep/")
    cmd(f"mkdir -p {keep_path} && mv {quote(str(video))} {quote(keep_path)}")


def watch(args):
    con = sqlite_con(args.db)

    bindings = []
    filename_include_sql = ""
    filename_exclude_sql = ""
    if args.search:
        for inc in args.search.split(","):
            filename_include_sql += " AND filename LIKE ? "
            bindings.append("%" + inc.replace(" ", "%").replace("%%", " ") + "%")
    if args.exclude:
        for exc in args.exclude.split(","):
            filename_exclude_sql += " AND filename NOT LIKE ? "
            bindings.append("%" + exc.replace(" ", "%").replace("%%", " ") + "%")

    sql_filter = conditional_filter(args)

    LIMIT = "LIMIT " + str(args.limit)
    OFFSET = f"OFFSET {args.skip}" if args.skip else ""

    query = f"""
    SELECT filename, duration/60/60 as hours, duration / size AS seconds_per_byte, size
    FROM media
    WHERE 1=1
    {filename_include_sql}
    {filename_exclude_sql}
    and {sql_filter}
    ORDER BY
            {args.sort + ',' if args.sort else ''}
            {'filename,' if args.print or args.search or args.play_in_order > 0 else ''}
            seconds_per_byte ASC
    {LIMIT} {OFFSET}
    """

    if args.aggregate:
        query = f"""select
            "Aggregate" as filename
            , sum(hours) hours
            , avg(seconds_per_byte) seconds_per_byte
            , sum(size) size
        from ({query})
        """

    if args.printquery:
        print_query(bindings, query)
        if args.play_in_order > 1:
            get_ordinal_media(con, args, Path("vid"), sql_filter)
        stop()

    if args.chromecast:
        args.cc_ip = get_ip_of_chromecast(args.device_name)

    if args.print:
        videos = pd.DataFrame([dict(r) for r in con.execute(query, bindings).fetchall()])

        if args.filename:
            if args.limit == 1:
                f = videos[["filename"]].loc[0].iat[0]
                if not Path(f).exists():
                    remove_media(con, f)
                    return main(args)
                print(f)
            else:
                csvf = videos[["filename"]].to_csv(index=False, header=False)
                print(csvf.strip())
        else:
            table_content = videos
            table_content[["filename"]] = table_content[["filename"]].applymap(
                lambda x: textwrap.fill(x, os.get_terminal_size().columns - 30)
            )
            table_content[["size"]] = table_content[["size"]].applymap(lambda x: humanize.naturalsize(x))
            print(
                tabulate(
                    table_content[["filename", "size", "hours"]],
                    tablefmt="fancy_grid",
                    headers="keys",
                    showindex=False,
                )
            )
            summary = videos.sum(numeric_only=True)
            duration = timedelta(hours=int(summary.hours), minutes=math.ceil(summary.hours % 1 * 60))
            print("Total duration:", humanize.precisedelta(duration, minimum_unit='minutes'))

        stop()

    if args.move:
        Path(args.move).mkdir(exist_ok=True, parents=True)
        keep_path = str(Path(args.move).resolve())

        videos = pd.DataFrame([dict(r) for r in con.execute(query, bindings).fetchall()])
        for video in videos[["filename"]]:
            if Path(video).exists() and "/keep/" not in video:
                quoted_next_video = quote(str(video))
                print(quoted_next_video)
                cmd(f"mv {quoted_next_video} {quote(keep_path)}")

            remove_media(con, video)
        stop()

    next_video = dict(con.execute(query, bindings).fetchone())["filename"]
    if args.play_in_order > 1:
        next_video = get_ordinal_media(con, args, Path(next_video), sql_filter)

    original_video = Path(next_video)
    next_video = Path(args.prefix + next_video)
    print(next_video)

    if "/keep/" in str(next_video):
        keep_video(next_video)
        remove_media(con, original_video)
        exit()

    if next_video.exists():
        quoted_next_video = quote(str(next_video))

        if args.only_video:
            has_video = (
                cmd(
                    f"ffprobe -show_streams -select_streams v -loglevel error -i {quoted_next_video} | wc -l",
                    quiet=True,
                ).stdout
                > "0"
            )
            if not has_video:
                remove_media(con, original_video)
                exit()

        if args.time_limit:  # TODO: replace with timer...
            seconds = args.time_limit * 60
            gap_time = 14
            temp_next_video = cmd(f"mktemp --suffix={next_video.suffix} --dry-run").stdout.strip()
            temp_video = cmd(f"mktemp --suffix={next_video.suffix} --dry-run").stdout.strip()

            # clip x mins of target video file into new temp video file for playback
            cmd(f"ffmpeg -i {quoted_next_video} -ss 0 -t {seconds} -c copy {temp_next_video}")
            # replace video file to prevent re-watching
            cmd(f"mv {quoted_next_video} {temp_video}")
            cmd(f"ffmpeg -i {temp_video} -ss {seconds - gap_time} -c copy {quoted_next_video} && rm {temp_video}")

            next_video = Path(temp_next_video)
            print(next_video)

        if args.force_transcode:
            temp_video = cmd(f"mktemp --suffix=.mkv --dry-run").stdout.strip()
            cmd(f"mv {quoted_next_video} {temp_video}")
            next_video = next_video.with_suffix(".mkv")
            cmd(
                (
                    f"ffmpeg -loglevel error -stats -i {temp_video} -map 0 -scodec webvtt -vcodec h264"
                    " -preset fast -profile:v high -level 4.1 -crf 17 -pix_fmt yuv420p"
                    " -acodec opus -ac 2 -b:a 128k -filter:a loudnorm=i=-18:lra=17"
                    f" {quoted_next_video} && rm {temp_video}"
                )
            )
            print(next_video)

        play_mpv(args, next_video)

        if args.keep and Confirm.ask("Keep?", default=False):
            keep_video(next_video)
        else:
            if len(args.prefix) > 0:
                cmd(f"/bin/rm {quoted_next_video}")
            else:
                cmd(f"trash-put {quoted_next_video}")

    remove_media(con, original_video)


def main():
    args = parse_args(default_chromecast="Living Room TV")

    watch(args)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['xklb']

package_data = \
{'': ['*']}

install_requires = \
['catt>=0.12.9,<0.13.0',
 'humanize>=4.2.3,<5.0.0',
 'ipython>=8.4.0,<9.0.0',
 'joblib>=1.1.0,<2.0.0',
 'mutagen>=1.45.1,<2.0.0',
 'natsort>=8.1.0,<9.0.0',
 'pandas>=1.4.3,<2.0.0',
 'protobuf<4',
 'rich>=12.5.1,<13.0.0',
 'sqlite-utils>=3.28,<4.0',
 'subliminal>=2.1.0,<3.0.0',
 'tabulate>=0.8.10,<0.9.0',
 'tinytag>=1.8.1,<2.0.0',
 'trash-cli>=0.22.4,<0.23.0']

entry_points = \
{'console_scripts': ['lb-extract = xklb.extract:main',
                     'lt = xklb.listen:main',
                     'wt = xklb.watch:main']}

setup_kwargs = {
    'name': 'xklb',
    'version': '1.12.4',
    'description': 'xk media library',
    'long_description': "## lb: opinionated media library\n\nRequires ffmpeg, mpv\n\n!!! You should be warned that the default action after watching a video is to trash it. Restore via your [trash can](https://specifications.freedesktop.org/trash-spec/trashspec-latest.html). Use `--keep` to ask y/n.\n\n### Install\n\n```\npip install xklb\n```\n\n### Step 1. Extract Metadata\n\n    lb-extract tv.db ./video/folder/\n\n    lb-extract --audio podcasts.db ./your/music/or/podcasts/folder/\n\n### Step 2. Watch / Listen\n\n    wt tv.db\n\n    lt podcasts.db\n\n### Repeat!\n\nImplementing repeat / auto-play is left to the end user. I recommend something like this if you use fish shell:\n\n```fish\nfunction repeat\n    while $argv\n        and :\n    end\nend\n\nrepeat lt audio.db\n```\n\nor\n\n```fish\nfunction repeatn --description 'repeatn <count> <command>'\n    for i in (seq 1 $argv[1])\n        eval $argv[2..-1]\n    end\nend\n\nrepeat 5 lt audio.db\n```\n\n#### Watch longest videos\n\n    wt tv.db --sort 'duration desc'\n\n#### Watch specific video series in order\n\n    wt tv.db --search 'title of series' --play-in-order\n\n#### There are multiple strictness levels of --play-in-order. If things aren't playing in order try adding more `O`s:\n\n    wt tv.db --search 'title of series' -O    # default\n    wt tv.db --search 'title of series' -OO   # slower, more complex algorithm\n    wt tv.db --search 'title of series' -OOO  # most strict\n\n#### I usually use the following:\n\n    lt -cast -s '  ost'      # for listening to OSTs on my chromecast groups\n    wt -u priority -w sub=0  # for exercising and watching YouTube\n    wt -u duration --print -s 'video title'  # when I want to check if I've downloaded something before\n\n#### Watch Options\n\n    wt -h\n\n    usage: watch.py [-h] [-1] [-cast-to CHROMECAST_DEVICE] [-cast] [-f] [-d DURATION]\n                    [-dM MAX_DURATION] [-dm MIN_DURATION] [-keep] [-list] [-filename]\n                    [-printquery] [-mv MOVE] [-O] [-r] [-s SEARCH] [-E EXCLUDE] [-S SKIP]\n                    [-t TIME_LIMIT] [-v] [-vlc] [-z SIZE] [-zM MAX_SIZE] [-zm MIN_SIZE]\n                    db\n\n    positional arguments:\n    db\n\n    options:\n    -h, --help            show this help message and exit\n    -1, --last\n    -cast-to CHROMECAST_DEVICE, --chromecast-device CHROMECAST_DEVICE\n    -cast, --chromecast\n    -f, --force-transcode\n    -d DURATION, --duration DURATION\n    -dM MAX_DURATION, --max-duration MAX_DURATION\n    -dm MIN_DURATION, --min-duration MIN_DURATION\n    -keep, --keep\n    -list, --list\n    -filename, --filename\n    -printquery, --printquery\n    -mv MOVE, --move MOVE\n    -O, --play-in-order\n    -r, --random\n    -s SEARCH, --search SEARCH\n    -E EXCLUDE, --exclude EXCLUDE\n    -S SKIP, --skip SKIP\n    -t TIME_LIMIT, --time-limit TIME_LIMIT\n    -v, --verbose\n    -vlc, --vlc\n    -z SIZE, --size SIZE\n    -zM MAX_SIZE, --max-size MAX_SIZE\n    -zm MIN_SIZE, --min-size MIN_SIZE\n\n#### Listen Options\n\n    lt -h\n\n    usage: listen.py [-h] [-cast] [-cast-to CHROMECAST_DEVICE] [-s SEARCH] [-E EXCLUDE]\n                    [-S SKIP] [-d DURATION] [-dm MIN_DURATION] [-dM MAX_DURATION]\n                    [-sz SIZE] [-szm MIN_SIZE] [-szM MAX_SIZE] [-mv MOVE] [-wl] [-O]\n                    [-r] [-v]\n                    db\n\n    positional arguments:\n    db\n\n    options:\n    -h, --help            show this help message and exit\n    -cast, --chromecast\n    -cast-to CHROMECAST_DEVICE, --chromecast-device CHROMECAST_DEVICE\n    -s SEARCH, --search SEARCH\n    -E EXCLUDE, --exclude EXCLUDE\n    -S SKIP, --skip SKIP\n    -d DURATION, --duration DURATION\n    -dm MIN_DURATION, --min-duration MIN_DURATION\n    -dM MAX_DURATION, --max-duration MAX_DURATION\n    -sz SIZE, --size SIZE\n    -szm MIN_SIZE, --min-size MIN_SIZE\n    -szM MAX_SIZE, --max-size MAX_SIZE\n    -mv MOVE, --move MOVE\n    -wl, --with-local\n    -O, --play-in-order\n    -r, --random\n    -v, --verbose\n",
    'author': 'Jacob Chapman',
    'author_email': '7908073+chapmanjacobd@users.noreply.github.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
